@@ -220,7 +220,21 @@ class ReportPDF:
         title, author, subject, keywords, outpath, path
 
     when using:
-        you must run report.pdf.close() after you run_qc(). """
+        you must run report.pdf.close() after you run_qc().
+
+    custom tables:
+        pass in arbitrary data using kwarg "custom_tables" as list with this structure:
+            'custom_tables': [
+                {
+                'title': "some title, optional",
+                'col_names': [list of strings],
+                'row_names': [list of strings, optional],
+                'data': [list of lists, with order matching col_names],
+                'order_after': [string name of the plot this should come after. It cannot appear first in list.]
+                },
+                {...second table here...}
+            ]
+    """
         # https://stackoverflow.com/questions/8187082/how-can-you-set-class-attributes-from-variable-arguments-kwargs-in-python
         self.__dict__.update(kwargs)
         self.__dict__['poobah_max_percent'] = self.__dict__.get('poobah_max_percent', 5)
@@ -281,6 +295,42 @@ Pre-processing pipeline
             # FUTURE lower priority: bis-conversion completeness, or GCT score --- https://rdrr.io/bioc/sesame/src/R/sesame.R
         }
 
+        self.custom = {}
+        if 'custom_tables' in self.__dict__:
+            self.parse_custom_tables(self.__dict__['custom_tables'])
+            LOGGER.info(f"Found custom_tables and inserted into order: {self.order}")
+            LOGGER.info(self.custom)
+
+    def parse_custom_tables(self, tables):
+        """tables is a list of {
+        'title': "some title, optional",
+        'col_names': [list of strings],
+        'row_names': [list of strings, optional],
+        'data': [list of lists, with order matching col_names],
+        'order_after': [string name of the plot this should come after. It cannot appear first in list.]
+        }"""
+        for table in tables:
+            required_attributes = {'title', 'data', 'order_after', 'col_names'}
+            for i in required_attributes:
+                if i not in table:
+                    raise KeyError("Your custom table must contain these keys: {required_attributes} (row_names is optional)")
+            #1 place order -- refer to this later in self.custom
+            try:
+                _index = self.order.index(table['order_after'])
+                self.order.insert(_index, table['title'])
+                self.custom[table['title']] = table
+            except ValueError:
+                raise ValueError(f"Your custom table's 'order_after' label is not in this list of chart objects, so could not be ordered: {self.order}")
+            if not isinstance(table['col_names'], list):
+                raise TypeError(f"Your custom table 'col_names' must be a list.")
+            if table.get('row_names') and not isinstance(table['row_names'], list):
+                raise TypeError(f"Your custom table 'row_names' must be a list.")
+            if not isinstance(table['data'], list):
+                raise TypeError(f"Your custom table 'data' must be a list of lists, matching the number of columns listed in col_names.")
+                if not (isinstance(table['data'][0],list) and len(table['data'][0]) == len(table['col_names'])):
+                    raise TypeError(f"Your custom table 'data' must be a list of lists, matching the number of columns listed in col_names.")
+            # passes! later, the table will be looked-up by title name and fed into chart in order.
+
     def open_error_buffer(self):
         """ preparing a stream of log messages to add to report appendix. """
         appendix = logging.getLogger()
@@ -347,7 +397,7 @@ Pre-processing pipeline
                     #pretty_table = Table(titles=['Sample_ID', 'MDS Pass/Fail'])
                     #for sample_id in df_indexes_to_retain:
 
-            elif part in self.plots:
+            elif part in self.plots and self.plots[part] == True:
                 if part == 'beta_density_plot':
                     LOGGER.info("Beta Density Plot")
                     fig = methylcheck.beta_density_plot(beta_df, save=False, silent=True, verbose=False, reduce=0.1, plot_title=None, ymax=None, return_fig=True)
@@ -376,6 +426,11 @@ Pre-processing pipeline
                     for fig in list_of_figs:
                         self.pdf.savefig(figure=fig, bbox_inches='tight')
                     self.plt.close('all')
+            elif part in self.custom:
+                table = self.custom[part]
+                self.to_table(table['data'], col_names=table['col_names'],
+                    row_names=table.get('row_names'), add_title=table['title'])
+
 
         # and finally, appendix of log messages
         appendix_msgs = self.errors.getvalue()
@@ -408,6 +463,30 @@ Pre-processing pipeline
         self.errors.close()
 
 
+    def exec_summary(self):
+        """QC exec summary
+    	sample_name/ID
+    	probe % failures
+    	probe_failure pass
+    	auto-qc result (only if present in kwargs passed in, otherwise omitted)
+    	MDS pass
+    	signal intensity pass
+    	if any fails, fail it (so overall pass)
+    table 2: meta
+    	array type (detect from data)
+    	number of samples (from data)
+    	processing pipeline version number (passed in)
+    	date processed (passed in)
+    	avg probe failure rate
+    	percent of samples that failed
+    	any failures from 'control probes'
+    	   reqs a way to capture warnings of data-off-chart
+        """
+        exec_summary_1 = []
+        exec_summary_2 = [] # list of lists
+        exec_summary_samples = [] # temp storage lookup
+        pass
+
     def page_of_text(self, text, pdf):
         """text is a single big string of text, with whitespace for line breaks.
         https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.text.html (0,0) is lower left; (1,1) is upper right """
@@ -422,6 +501,7 @@ Pre-processing pipeline
         pdf.savefig()
         self.plt.close()
 
+
     def page_of_paragraphs(self, para_list, pdf, line_height='double'):
         """ https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.text.html (0,0) is lower left; (1,1) is upper right.
         this version estimates the size of each paragraph and moves the origin downward accordingly.
@@ -480,9 +560,9 @@ Pre-processing pipeline
         - attempts to split long tables into multiple pages.
         - should warn if table is too wide to fit. """
 
-        # stringify numbers
+        # stringify numbers; replace None
         for idx, sub in enumerate(list_of_lists):
-            list_of_lists[idx] = [str(i) for i in sub]
+            list_of_lists[idx] = [('' if isinstance(i,type(None)) else str(i)) for i in sub]
 
         rows_per_page = 42 # appears to work for standard default PDF page size and font size.
         pages = math.ceil(len(list_of_lists)/rows_per_page)
